function [objective_function_p_cand] = compute_GCCPHAT_cost_function_efficient(p_array,x_bar,w,p,c)
% compute_GCCPHAT_cost_function_efficient - Compute GCC-PHAT cost function
%
% Other m-files required: none
% Subfunctions: none
% MAT-files required: none
%
% Based on the paper:
%			 "Direct Localization in Underwater Acoustics via Convolutional 
%             Neural Networks: A Data-Driven Approach", Amir Weiss,
%             Toros Arikan, and Gregory W. Wornell,
%			  in Proc. of MLSP, August 2022.
%
% See also: supplementary_materials_DLOC_MLSP2022.pdf for more information about this MATLAB package
% Author: Amir Weiss
% email: amirweiss15@gmail.com
% Website: http://www.weissamir.com
% July 2022; Last revision: 11-Jul-2022
%------------- BEGIN CODE --------------

[T,L] = size(x_bar);
x_cand = p(1);
y_cand = p(2);
z_cand = p(3);
x_receiver = p_array(1,:)';
y_receiver = p_array(2,:)';
z_receiver = p_array(3,:)';

x_shifted_bar = zeros(T,L);

for l=1:L
    distance_source_reciver_cand = sqrt( (x_cand-x_receiver(l)).^2 + (y_cand-y_receiver(l)).^2 + (z_cand-z_receiver(l)).^2 );
    tau_cand = distance_source_reciver_cand/c;
    x_shifted_bar(:,l) = x_bar(:,l).*exp(1i*tau_cand*w');
end

objective_function_p_cand = 0;

for l1=1:(L-1)
    for l2=(l1+1):L
        cross_corr_l1_l2 = x_shifted_bar(:,l1).*conj(x_shifted_bar(:,l2));
        magnitude_cross_corr_l1_l2 = abs(cross_corr_l1_l2);
        GCCPHAT_corr_bar = cross_corr_l1_l2./magnitude_cross_corr_l1_l2;
        GCCPHAT_corr = ifft(GCCPHAT_corr_bar);
        
        objective_function_p_cand = objective_function_p_cand + abs(GCCPHAT_corr(1));
    end
end


end
%------------- END OF CODE --------------