function [SBL_val,MFP_val] = compute_SBL_and_oracleMFP3_objective_function(p_array,x_bar,h,w,p,c,b_MFP3oracle)
% compute_SBL_and_oracleMFP3_objective_function - Compute the SBL and "oracle" MFP objective functions
%
% Other m-files required: none
% Subfunctions: none
% MAT-files required: none
%
% Based on the paper:
%			 "Direct Localization in Underwater Acoustics via Convolutional 
%             Neural Networks: A Data-Driven Approach", Amir Weiss,
%             Toros Arikan, and Gregory W. Wornell,
%			  in Proc. of MLSP, August 2022.
%
% See also: supplementary_materials_DLOC_MLSP2022.pdf for more information about this MATLAB package
% Author: Amir Weiss
% email: amirweiss15@gmail.com
% Website: http://www.weissamir.com
% July 2022; Last revision: 11-Jul-2022
%------------- BEGIN CODE --------------

[T,L] = size(x_bar);
x_cand = p(1);
y_cand = p(2);
z_cand = p(3);
x_receiver = p_array(1,:)';
y_receiver = p_array(2,:)';
z_receiver = p_array(3,:)';
Gamma = zeros(T,3,L);
Q_block = [];
h_p = zeros(T,L);
for l_ind=1:L
    %%%%%%%%
    distance_source_reciver_cand = sqrt( (x_cand-x_receiver(l_ind)).^2 + (y_cand-y_receiver(l_ind)).^2 + (z_cand-z_receiver(l_ind)).^2 );
    tau1_cand = distance_source_reciver_cand/c;
    horizontal_distance = sqrt( (x_cand-x_receiver(l_ind)).^2 + (y_cand-y_receiver(l_ind)).^2 );
    distance_surface_bounce_cand = sqrt ( horizontal_distance^2 + (z_cand + z_receiver(l_ind))^2 );
    tau2_cand = distance_surface_bounce_cand/c;
    distance_bottom_bounce_cand = sqrt ( horizontal_distance^2 + (2*h - z_cand - z_receiver(l_ind))^2 );
    tau3_cand = distance_bottom_bounce_cand/c;
    
    tau_cand = [tau1_cand ; tau2_cand ; tau3_cand];
    
    D_l = transp(exp(-1i*tau_cand*w));
    
    % For DPD
    try
        L_l = chol(D_l'*D_l);
    catch
        L_l = chol(D_l'*D_l + (1e-10)*eye(3));
    end
    Gamma(:,:,l_ind) = (conj(x_bar(:,l_ind)).*D_l)/L_l;
    Q_block = [Q_block Gamma(:,:,l_ind)];
    
    % For MFP
    h_p(:,l_ind) = D_l*b_MFP3oracle(:,l_ind);
    %%%%%%%%
end

SBL_val = max(real(eig(Q_block'*Q_block)));

MFP_val = 0;
for k=1:T
    MFP_val = MFP_val + ( abs(conj(x_bar(k,:))*transp(h_p(k,:)))^2 )/( norm(h_p(k,:))^2 );
end

end
%------------- END OF CODE --------------