# Model for DOA estimation
# Based on the paper:
#			 "Direct Localization in Underwater Acoustics via Convolutional 
#             Neural Networks: A Data-Driven Approach", Amir Weiss,
#             Toros Arikan, and Gregory W. Wornell,
#			  in Proc. of MLSP, August 2022.
#
# See also: supplementary_materials_DLOC_MLSP2022.pdf for more information about this Python package
# Author: Amir Weiss
# email: amirweiss15@gmail.com
# Website: http://www.weissamir.com
# July 2022; Last revision: 11-Jul-2022
#------------- BEGIN CODE --------------
import tensorflow as tf
import tensorflow.keras as k
from tensorflow.keras import layers
from tensorflow.keras.models import Model

import numpy as np
import matplotlib.pyplot as plt
from tqdm import tqdm
import time
import pickle
import os


def get_custom_model_DOA_est_MCE(input_shape, T, k_sz=3, long_k_sz=101, lr=0.0003):

    start_neurons = 8

    num_of_functions = input_shape[0]

    in0 = layers.Input(shape=input_shape)
    x = in0
    x = layers.BatchNormalization()(x)

    # 200
    conv1 = layers.Conv2D(T, (num_of_functions, long_k_sz), activation="relu", padding="same")(x)
    conv1 = layers.Conv2D(T, (num_of_functions, long_k_sz), activation="relu", padding="same")(conv1)
    conv1 = layers.Conv2D(start_neurons * 8, (num_of_functions, k_sz), activation="relu", padding="same")(conv1)
    pool1 = layers.AveragePooling2D((1, 2))(conv1)
    pool1 = layers.Dropout(0.5)(pool1)

    # 100
    conv2 = layers.Conv2D(start_neurons * 8, (k_sz, k_sz), activation="relu", padding="same")(pool1)
    conv2 = layers.Conv2D(start_neurons * 8, (k_sz, k_sz), activation="relu", padding="same")(conv2)
    pool2 = layers.AveragePooling2D((1, 2))(conv2)
    pool2 = layers.Dropout(0.5)(pool2)

    # 50
    conv3 = layers.Conv2D(start_neurons * 4, (k_sz, k_sz), activation="relu", padding="same")(pool2)
    conv3 = layers.Conv2D(start_neurons * 4, (k_sz, k_sz), activation="relu", padding="same")(conv3)
    pool3 = layers.AveragePooling2D((1, 2))(conv3)
    pool3 = layers.Dropout(0.5)(pool3)


    # 25
    conv4 = layers.Conv2D(start_neurons * 2, (k_sz, k_sz), activation="relu", padding="same")(pool3)
    conv4 = layers.Conv2D(start_neurons * 2, (k_sz, k_sz), activation="relu", padding="same")(conv4)
    pool4 = layers.AveragePooling2D((1, 2))(conv4)
    pool4 = layers.Dropout(0.5)(pool4)

    # 12
    convm = layers.Conv2D(start_neurons * 8, (k_sz, k_sz), activation="relu", padding="same")(pool4)
    convm = layers.Conv2D(start_neurons * 8, (k_sz, k_sz), activation="relu", padding="same")(convm)
    poolm = layers.AveragePooling2D((1, 2))(convm)

    # 3
    two_before_output_layer = layers.Flatten()(poolm)
    one_before_output_layer = layers.Dense(10,activation='relu')(two_before_output_layer)
    output_layer = layers.Dense(1,activation='relu')(one_before_output_layer)
    x_out = output_layer

    supreg_net = Model(in0, x_out, name='supreg')

    supreg_net.compile(optimizer=k.optimizers.Adam(learning_rate=lr),
              loss=my_loss_fn,
              metrics=[tf.keras.losses.MeanSquaredError()])
    
    return supreg_net

def my_loss_fn(y_true, y_pred):
    cyclic_error = 2 - 2*tf.math.cos(y_pred - y_true)
    return tf.reduce_mean(cyclic_error, axis=-1)  # Note the `axis=-1`
#------------- END CODE --------------